import requests

from social_user_info import status_codes
from social_user_info.constants import DEFAULT_TIMEOUT, FACEBOOK_PROFILE_FIELDS, FACEBOOK_PROFILE_PIC_API_URL, \
    FACEBOOK_PROFILE_API_URL
from social_user_info.social_medias.abstract_social_media import AbstractSocialMediaAPI


class FacebookAPI(AbstractSocialMediaAPI):
    """
    This class is used for getting user information from facebook using facebook api and the user's access token
    """
    PROFILE_URL = FACEBOOK_PROFILE_API_URL
    PROFILE_PIC_URL = FACEBOOK_PROFILE_PIC_API_URL

    @staticmethod
    def get_profile_api_params(access_token):
        return {
            'access_token': access_token,
            'fields': FACEBOOK_PROFILE_FIELDS
        }

    @staticmethod
    def get_profile_pic_api_params(access_token, height=100, width=100):
        return {
            'access_token': access_token,
            'redirect': 'false',
            'height': height,
            'width': width
        }

    @classmethod
    def get_profile_picture_url(cls, access_token):
        profile_pic_api = requests.get(
            url=cls.PROFILE_PIC_URL, params=cls.get_profile_pic_api_params(access_token), timeout=DEFAULT_TIMEOUT
        )

        return {
            **{'profile_picture': profile_pic_api.json().get('data', {}).get('url')},
            **cls.get_response_status(profile_pic_api.status_code)
        }

    @classmethod
    def get_user_info(cls, access_token):
        profile_api = requests.get(
            url=cls.PROFILE_URL, params=cls.get_profile_api_params(access_token), timeout=DEFAULT_TIMEOUT
        )

        if profile_api.status_code == status_codes.HTTP_OK_REQUEST:
            user_info = {
                **profile_api.json(),
                **cls.get_profile_picture_url(access_token)
            }
        else:
            user_info = {
                **profile_api.json(),
                **cls.get_response_status(profile_api.status_code)
            }

        return user_info



class AbstractSocialMediaAPI:
    class Meta:
        abstract = True

    @staticmethod
    def get_response_status(status_code):
        return {'status': status_code}

    @classmethod
    def get_user_info(cls, access_token):
        raise NotImplementedError('get_user_info() must be implemented')



import requests

from social_user_info import status_codes
from social_user_info.constants import GOOGLE_PROFILE_API_URL, AUTHORIZATION_KEY, AUTHORIZATION_VALUE, DEFAULT_TIMEOUT
from social_user_info.social_medias.abstract_social_media import AbstractSocialMediaAPI


class GoogleAPI(AbstractSocialMediaAPI):
    """
    This class is used for getting user information from google using google api and the user's access token
    """
    PROFILE_URL = GOOGLE_PROFILE_API_URL

    @staticmethod
    def get_authorization_header(access_token):
        return {AUTHORIZATION_KEY: AUTHORIZATION_VALUE.format(access_token=access_token)}

    @staticmethod
    def get_transformed_keys(response):
        return {
            **response,
            'first_name': response.pop('given_name', None),
            'last_name': response.pop('family_name', None),
            'profile_picture': response.pop('picture', None)
        }

    @classmethod
    def get_user_info(cls, access_token):
        profile_api = requests.get(
            url=cls.PROFILE_URL, headers=cls.get_authorization_header(access_token), timeout=DEFAULT_TIMEOUT
        )

        if profile_api.status_code == status_codes.HTTP_OK_REQUEST:
            user_info = {
                **cls.get_transformed_keys(profile_api.json()),
                **cls.get_response_status(profile_api.status_code)
            }
        else:
            user_info = {
                **profile_api.json(),
                **cls.get_response_status(profile_api.status_code)
            }

        return user_info



from social_user_info import constants
from social_user_info import status_codes
from social_user_info.social_medias import GoogleAPI, FacebookAPI, GitHubAPI, MicrosoftAPI


class APIService:
    SOURCE_TO_SOCIAL_API = {
        constants.AUTH_SOURCE_GOOGLE: GoogleAPI,
        constants.AUTH_SOURCE_FACEBOOK: FacebookAPI,
        constants.AUTH_SOURCE_GITHUB: GitHubAPI,
        constants.AUTH_SOURCE_MICROSOFT: MicrosoftAPI
    }

    @classmethod
    def get_user_info(cls, access_token, auth_source):
        """
        Method of a util class that delegates the work for getting user info to the required SocialMedia
        Service based on the incoming auth source
        :param access_token:
        :param auth_source:
        :return: user_info
        """
        user_info = {
            'status': status_codes.HTTP_BAD_REQUEST,
            'error': constants.INVALID_AUTH_SOURCE_ERROR.format(auth_source)
        }

        api_class = cls.get_api(auth_source)
        if api_class:
            user_info = api_class.get_user_info(access_token)
        return user_info

    @classmethod
    def get_api(cls, auth_source):
        """
        Method that returns the social media api class corresponding to source
        :param auth_source:
        :return:
        """
        return cls.SOURCE_TO_SOCIAL_API.get(auth_source)



import requests

from social_user_info.constants import MICROSOFT_API_URL, AUTHORIZATION_KEY, AUTHORIZATION_VALUE, DEFAULT_TIMEOUT
from social_user_info.social_medias.abstract_social_media import AbstractSocialMediaAPI


class MicrosoftAPI(AbstractSocialMediaAPI):
    """
        This class is used for getting user information from microsoft using microsoft api and the user's access token
    """
    PROFILE_URL = MICROSOFT_API_URL

    @staticmethod
    def get_authorization_header(access_token):
        return {AUTHORIZATION_KEY: AUTHORIZATION_VALUE.format(access_token=access_token)}

    @staticmethod
    def get_profile_picture():
        # In version 1.0 of microsoft, profile picture is not supported
        return {'profile_picture': None}

    @classmethod
    def get_transformed_keys(cls, profile_info):
        return {
            **profile_info,
            'first_name': profile_info.pop('givenName', None),
            'last_name': profile_info.pop('surname', None),
            'email': profile_info.pop('userPrincipalName', None),
            **cls.get_profile_picture()
        }

    @classmethod
    def get_user_info(cls, access_token):
        profile_api = requests.get(
            url=cls.PROFILE_URL, headers=cls.get_authorization_header(access_token), timeout=DEFAULT_TIMEOUT
        )

        if profile_api:
            user_info = {
                **cls.get_transformed_keys(profile_api.json()),
                **cls.get_response_status(profile_api.status_code)
            }
        else:
            user_info = {
                **profile_api.json(),
                **cls.get_response_status(profile_api.status_code)
            }

        return user_info



# ------------------------------------------ API URLs ------------------------------------------------------------------

GOOGLE_PROFILE_API_URL = 'https://www.googleapis.com/userinfo/v2/me'
FACEBOOK_PROFILE_API_URL = 'https://graph.facebook.com/v3.2/me'
FACEBOOK_PROFILE_PIC_API_URL = 'https://graph.facebook.com/v3.2/me/picture'
GITHUB_PROFILE_API_URL = 'https://api.github.com/user'
GITHUB_EMAIL_API_URL = 'https://api.github.com/user/emails'
MICROSOFT_API_URL = 'https://graph.microsoft.com/v1.0/me/'

# ------------------------------------------ Service Constants ---------------------------------------------------------

AUTHORIZATION_KEY = 'Authorization'
AUTHORIZATION_VALUE = 'Bearer {access_token}'
FACEBOOK_PROFILE_FIELDS = 'email,first_name,last_name'

DEFAULT_TIMEOUT = 5  # 5 Seconds

# ------------------------------------------ Auth Source Constants -----------------------------------------------------

AUTH_SOURCE_GOOGLE = 'google'
AUTH_SOURCE_FACEBOOK = 'facebook'
AUTH_SOURCE_GITHUB = 'github'
AUTH_SOURCE_MICROSOFT = 'microsoft'

INVALID_AUTH_SOURCE_ERROR = "{} source is not supported in this service"

# ----------------------------------------------------------------------------------------------------------------------



import requests

from social_user_info import status_codes
from social_user_info.constants import DEFAULT_TIMEOUT, GITHUB_PROFILE_API_URL, AUTHORIZATION_KEY, AUTHORIZATION_VALUE, \
    GITHUB_EMAIL_API_URL
from social_user_info.social_medias.abstract_social_media import AbstractSocialMediaAPI


class GitHubAPI(AbstractSocialMediaAPI):
    """
    This class is used for getting user information from facebook using facebook api and the user's access token
    """
    PROFILE_URL = GITHUB_PROFILE_API_URL
    EMAIL_URL = GITHUB_EMAIL_API_URL

    @staticmethod
    def get_authorization_header(access_token):
        return {AUTHORIZATION_KEY: AUTHORIZATION_VALUE.format(access_token=access_token)}

    @staticmethod
    def get_transformed_keys(response):
        return {
            **response,
            'first_name': response.get('name', '').split(' ')[0],
            'last_name': response.pop('name', '').split(' ')[-1],
            'profile_picture': response.pop('avatar_url', None)
        }

    @classmethod
    def get_user_email(cls, access_token):
        email_api = requests.get(
            url=cls.EMAIL_URL, headers=cls.get_authorization_header(access_token), timeout=DEFAULT_TIMEOUT
        )

        response = {'email': None}
        if email_api.status_code == status_codes.HTTP_OK_REQUEST:
            private_emails = email_api.json()

            response['email'] = private_emails[0]['email']
            for private_email in private_emails:
                if private_email['primary']:
                    response['email'] = private_email['email']

        return response

    @classmethod
    def get_user_info(cls, access_token):
        profile_api = requests.get(
            url=cls.PROFILE_URL, headers=cls.get_authorization_header(access_token), timeout=DEFAULT_TIMEOUT
        )

        if profile_api.status_code == status_codes.HTTP_OK_REQUEST:
            user_info = {
                **cls.get_transformed_keys({**profile_api.json(), **cls.get_user_email(access_token)}),
                **cls.get_response_status(profile_api.status_code)
            }
        else:
            user_info = {
                **profile_api.json(),
                **cls.get_response_status(profile_api.status_code)
            }

        return user_info
